//+------------------------------------------------------------------+
//|                                               Harami Boy Boy.mq5 |
//|                                       Copyright 2021, Brian Jaka |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, Brian Jaka"
#property link      "https://www.mql5.com"
#property version   "1.00"
/*
 This EA trades a Harami setup at open of Bar[0]
 BUY - One bearish and one bullish bar formation
 SELL- One bullish and one bearish bar formation
*/
#include<Trade\SymbolInfo.mqh>
#include<Trade\Trade.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>
//--- 
CSymbolInfo    m_symbol;                     // object of CSymbolInfo class for receiving symbol settings
CTrade         m_trade;                      // object of CTrade class for performing trade operations
CPositionInfo  m_position;                   // object of CPositionInfo class for receiving position settings
CAccountInfo   m_account;                    // object of CAccountInfo class for receiving account settings
//--- input parameters
input group             "Expert Settings"
input int      InpTakeProfitPts     =  20000;   //Take profit points
input int      InpStopLossPts       =  10000;   //Stop loss points
input double   InpOrderSize         =  0.20;    //Order size
input bool     InpPrintLog          =  false;   // Print log
input ulong    InpDeviation         =  10;      // Deviation
input ulong    InpMagic             =  12345;   // Magic number
//---
bool     m_need_close_buys          = false;    // close all BUY positions
bool     m_need_close_sells         = false;    // close all SELL positions
bool     m_need_open_buy            = false;    // open BUY position
bool     m_need_open_sell           = false;    // open SELL position
datetime m_prev_bars                = 0;        // "0" -> D'1970.01.01 00:00';
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- set the name for the appropriate symbol
   ResetLastError();
   if(!m_symbol.Name(Symbol())) // sets symbol name
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: CSymbolInfo.Name");
      return(INIT_FAILED);
     }
//---
   m_trade.SetExpertMagicNumber(InpMagic);            //--- set magicNumber for your orders identification
   m_trade.SetMarginMode();                           //--- set margin mode for your orders 
   m_trade.SetTypeFillingBySymbol(m_symbol.Name());   //--- set order filling mode allowed by the server
   m_trade.SetDeviationInPoints(InpDeviation);        //--- set available slippage in points when buying/selling
//--- receive current rates and display
   m_symbol.RefreshRates();
   Print(m_symbol.Name()," (",m_symbol.Description(),")",
         "  Bid=",m_symbol.Bid(),"   Ask=",m_symbol.Ask());
//--- receive minimum freeze levels for trade operations
   Print("StopsLevel=",m_symbol.StopsLevel()," points, FreezeLevel=",
         m_symbol.FreezeLevel()," points");
//--- receive the number of decimal places and point size
   Print("Digits=",m_symbol.Digits(),
         ", Point=",DoubleToString(m_symbol.Point(),m_symbol.Digits()));
//--- spread info
   Print("SpreadFloat=",m_symbol.SpreadFloat(),", Spread(current)=",
         m_symbol.Spread()," points");
//--- request order execution type for limitations
   Print("Limitations for trade operations: ",EnumToString(m_symbol.TradeMode()),
         " (",m_symbol.TradeModeDescription(),")");
//--- clarifying trades execution mode
   Print("Trades execution mode: ",EnumToString(m_symbol.TradeExecution()),
         " (",m_symbol.TradeExecutionDescription(),")");
//--- clarifying contracts price calculation method
   Print("Contract price calculation: ",EnumToString(m_symbol.TradeCalcMode()),
         " (",m_symbol.TradeCalcModeDescription(),")");
//--- sizes of contracts
   Print("Standard contract size: ",m_symbol.ContractSize(),
         " (",m_symbol.CurrencyBase(),")");
//--- minimum and maximum volumes in trade operations
   Print("Volume info: LotsMin=",m_symbol.LotsMin(),"  LotsMax=",m_symbol.LotsMax(),
         "  LotsStep=",m_symbol.LotsStep());
//--- 
   Print(__FUNCTION__,"  completed");
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- 
   double Ask=NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_ASK),_Digits);  // calculate the Ask price
   double Bid=NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_BID),_Digits);  // calculate the Bid price
   string signal  ="";                                                        // Create a string for the signal
   int    digits  =(int)SymbolInfoInteger(_Symbol,SYMBOL_DIGITS);             // number of decimal places  
   double point   =SymbolInfoDouble(_Symbol,SYMBOL_POINT);                    // point
   double SellSL  =Bid-InpStopLossPts*point;                                  // unnormalized SL value
          SellSL  =NormalizeDouble(SellSL,digits);                            // normalizing Stop Loss 
   double SellTP  =Bid+InpTakeProfitPts*point;                                // unnormalized TP value
          SellTP  =NormalizeDouble(SellTP,digits);                            // normalizing Take Profit 
   double BuySL   =Ask-InpStopLossPts*point;                                  // unnormalized SL value
          BuySL   =NormalizeDouble(BuySL,digits);                             // normalizing Stop Loss 
   double BuyTP   =Ask+InpTakeProfitPts*point;                                // unnormalized TP value
          BuyTP   =NormalizeDouble(BuyTP,digits);                              // normalizing Take Profit

//--- work only at the time of the birth of new bar
   datetime time_0=iTime(m_symbol.Name(),Period(),0);
   if(time_0==m_prev_bars)
      return;
   m_prev_bars=time_0;           
//--- Condition for opening BUY position
   if(m_need_open_buy)
//-- Check for Harami signal and open buy on Bar[0]
   if ((iOpen(m_symbol.Name(), 15, 1) - iClose(m_symbol.Name(), 15, 1)) < 0) { // 1st candle before current must be bullish
   if ((iOpen(m_symbol.Name(), 15, 2) - iClose(m_symbol.Name(), 15, 2)) > 0) { // 2nd candle before current must be bearish
    {
     m_need_open_buy=true;
      return;
     }      
//--- check volume before OrderSend to avoid "not enough money" error (CTrade)
      double free_margin_check=m_account.FreeMarginCheck(m_symbol.Name(),
                               ORDER_TYPE_BUY,
                               m_symbol.LotsMin(),
                               m_symbol.Ask());
      double margin_check=m_account.MarginCheck(m_symbol.Name(),
                          ORDER_TYPE_BUY,
                          m_symbol.LotsMin(),
                          m_symbol.Ask());
      if(free_margin_check>margin_check)
        {
         if(InpPrintLog)
            Print(__FILE__," ",__FUNCTION__,", OK: ","Signal BUY");
         m_trade.Buy(m_symbol.LotsMin(),m_symbol.Name(),m_symbol.Ask(),BuySL,BuyTP);
        }
      m_need_open_buy=false;
     }   
//--- Condition for opening SELL position
   if(m_need_open_sell)
//-- Check for Bearish signal and open sell on Bar[0] 
   if ((iOpen(m_symbol.Name(), 15, 1) - iClose(m_symbol.Name(), 15, 1)) > 0) { // 1st candle before current must be bearish
   if ((iOpen(m_symbol.Name(), 15, 2) - iClose(m_symbol.Name(), 15, 2)) < 0) { // 2nd candle before current must be bullish
    {
     m_need_open_sell=true;
      return;
     }   
//--- check volume before OrderSend to avoid "not enough money" error (CTrade)
      double free_margin_check=m_account.FreeMarginCheck(m_symbol.Name(),
                               ORDER_TYPE_SELL,
                               m_symbol.LotsMin(),
                               m_symbol.Ask());
      double margin_check=m_account.MarginCheck(m_symbol.Name(),
                          ORDER_TYPE_SELL,
                          m_symbol.LotsMin(),
                          m_symbol.Ask());
      if(free_margin_check>margin_check)
        {
         if(InpPrintLog)
            Print(__FILE__," ",__FUNCTION__,", OK: ","Signal SELL");
         m_trade.Sell(m_symbol.LotsMin(),m_symbol.Name(),m_symbol.Bid(),SellSL,SellTP);
        
        }
      m_need_open_sell=false;
        }
      }
    }         
  }
//+------------------------------------------------------------------+
